/*

	Part of the upgraded dcutils by Donald Haase.

	I wrote this to try to probe maple devices for 
	information. The goal is to gather up this 
	information in order to hopefully find distinctions 
	between devices we normally identify as being the 
	same, such as third party peripherals.

*/
#include <kos.h>
#include <dc/maple.h>

#define GRABALL 2
/* Local copy of the returned buffer for maple stuff */
unsigned char recv_buff[1024 + 32];

static void vbl_allinfo_callback(maple_frame_t * frm) {
	maple_response_t	*resp;

	/* So.. did we get a response? */
	resp = (maple_response_t *)frm->recv_buf;

	if (resp->response == MAPLE_RESPONSE_ALLINFO) {
		printf("Received proper maple response\n");
		/* Copy in the new buff */
		memcpy(recv_buff, resp, 1024 + 32);
	} else {
		printf("maple: bad response %d on device\n",resp->response); 
	}
	maple_frame_unlock(frm);
}

/* Send a ALLINFO command for the given port/unit */
static void vbl_send_allinfo(int p, int u) {
	maple_device_t * dev;

	dev = &maple_state.ports[p].units[u];

	/* Reserve access; if we don't get it, forget about it */
	if (maple_frame_lock(&dev->frame) < 0)
		return;

	/* Setup our autodetect frame to probe at a new device */
	maple_frame_init(&dev->frame);
	dev->frame.cmd = MAPLE_COMMAND_ALLINFO;
	dev->frame.dst_port = p;
	dev->frame.dst_unit = u;
	dev->frame.callback = vbl_allinfo_callback;
	maple_queue_frame(&dev->frame);
}

/* Formatted output for the devinfo struct */
void print_devinfo(maple_devinfo_t *info)
{
	/* Print each piece of info in an itemized fashion */	
	printf("Function int 0: %.8lx\n", info->functions);
	printf("Function int 1: %.8lx\n", info->function_data[0]);
	printf("Function int 2: %.8lx\n", info->function_data[1]);
	printf("Function int 3: %.8lx\n", info->function_data[2]);
	printf("Region Code: %.2x\n", info->area_code);
	printf("Connection : %.2x\n", info->connector_direction);
	printf("Product Name: %.30s\n", info->product_name);
	printf("Product License: %.60s\n", info->product_license);
	printf("Standby current consumption: %.4x\n", info->standby_power);
	printf("Maximum current consumption: %.4x\n", info->max_power);
	printf("\n");
}

#define BPL 16	/* Bytes to print per line */
/* 
	Get and print the data returned by the ALLINFO command, 
	if extra=0, do not print the Free Data
*/
void print_device_allinfo(maple_device_t *dev, int extra)
{
	unsigned int size = 4;
	unsigned int i = 0;
	unsigned int j = 0;		

	if(dev == NULL)	return;	

	/* Clear the old buffer */
	memset(recv_buff, 0, 1024+32);

	printf("Trying to get the allinfo for device %c%c:\n", 'A'+(dev->port), '0'+(dev->unit));	
	vbl_send_allinfo((dev->port), (dev->unit));
	timer_spin_sleep(800);
	size += (recv_buff[3]*4);
	printf("Recieved %d bytes of data. Here it is: \n", size-4);	
	print_devinfo((maple_devinfo_t *)&recv_buff[4]);

	if(!extra) return;

#if (GRABALL == 1)
	size = 1024+32;
#elif (GRABALL == 2)
	/* Grab a bit extra, cause some devices are stupid, and give more than they tell */
	size = ((size+64)<(1024+32))?(size+64):(1024+32);
#endif

	printf("Extra data: \n");
	/* Print the hex for the data followed by ascii */
	for(i=4+112; i< size; i+=BPL) 
	{	
		printf("Extra+0x%.3x | ", i-(4+112));
		for(j=0;j<BPL;j++)	printf("%.2x ", recv_buff[i+j]);		
		printf("| ");
		
		for(j=0;j<BPL;j++)	printf("%c"   , isprint(recv_buff[i+j])?recv_buff[i+j]:' ');
		printf("\n");
	}	
	printf("End of Extra data\n\n");		
}

/* 
	Get the total number of maple devices, 
	then perform print_device_allinfo on each
*/
void print_all_device_allinfo(int extra)
{
	int num = maple_enum_count();
	while(num--)		
		print_device_allinfo(maple_enum_type(num, 0xffffffff), extra);	
}

int main (void)
{
	printf("Going to print info on all devices (in reverse order): \n");
	print_all_device_allinfo(1);
	printf("That's all the devices attached.\n");
	return 0;
}

